@@ -4,6 +4,7 @@
 #
 # Copyright (C) 1998 Christian Schwarz and Richard Braakman
 # Copyright (C) 2007 Russ Allbery
+# Copyright (C) 2017-2019 Chris Lamb <lamby@debian.org>
 #
 # This program is free software.  It is distributed under the terms of
 # the GNU General Public License as published by the Free Software
@@ -17,10 +18,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, you can find it on the World Wide
-# Web at http://www.gnu.org/copyleft/gpl.html, or write to the Free
+# Web at https://www.gnu.org/copyleft/gpl.html, or write to the Free
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA.
 
+package reporting_html_reports;
+
 use v5.20;
 use warnings;
 use utf8;
@@ -37,15 +40,14 @@ use Text::Template ();
 use URI::Escape;
 use YAML::XS ();
 
-use Lintian::Command qw(safe_qx);
 use Lintian::Data;
-use Lintian::Deb822Parser qw(read_dpkg_control);
-use Lintian::Internal::FrontendUtil qw(split_tag);
+use Lintian::Deb822::Parser qw(read_dpkg_control_lc);
+use Lintian::IO::Async qw(safe_qx);
 use Lintian::Profile;
 use Lintian::Relation::Version qw(versions_comparator);
 use Lintian::Reporting::ResourceManager;
 use Lintian::Reporting::Util qw(load_state_cache find_backlog);
-use Lintian::Util qw(copy_dir run_cmd check_path);
+use Lintian::Util qw(copy_dir run_cmd locate_executable);
 
 my $CONFIG;
 my %OPT;
@@ -58,7 +60,7 @@ my %OPT_HASH = ('reporting-config=s'=> \$OPT{'reporting-config'},);
 my (
     $RESOURCE_MANAGER, $LINTIAN_VERSION, $timestamp,
     $TEMPLATE_CONFIG_VARS,$HARNESS_STATE_DIR, $HISTORY_DIR,
-    $HISTORY, $GRAPHS, $LINTIAN_ROOT,
+    $HISTORY, $GRAPHS, $LINTIAN_BASE,
     $HTML_TMP_DIR, $SCOUR_ENABLED,
 );
 # FIXME: Should become obsolete if gnuplot is replaced by R like piuparts.d.o /
@@ -130,6 +132,23 @@ my @attrs = qw(maintainers source-packages binary-packages udeb-packages
   errors warnings info experimental pedantic overridden groups-known
   groups-backlog classifications groups-with-errors);
 
+my @RESTRICTED_CONFIG_DIRS= split(/:/, $ENV{'LINTIAN_RESTRICTED_CONFIG_DIRS'});
+my @CONFIG_DIRS = split(/:/, $ENV{'LINTIAN_CONFIG_DIRS'});
+
+sub load_profile {
+    my ($profile_name, $options) = @_;
+    my %opt = (
+        'restricted-search-dirs' => \@RESTRICTED_CONFIG_DIRS,
+        %{$options // {}},
+    );
+    require Lintian::Profile;
+
+    my $profile = Lintian::Profile->new;
+    $profile->load($profile_name, \@CONFIG_DIRS, \%opt);
+
+    return $profile;
+}
+
 sub required_cfg_value {
     my (@keys) = @_;
     my $v = $CONFIG;
@@ -161,14 +180,14 @@ sub main {
 
     setup_output_dir(
         'output_dir'       => $HTML_TMP_DIR,
-        'lintian_manual'   => "${LINTIAN_ROOT}/doc/lintian.html",
-        'lintian_api_docs' => "${LINTIAN_ROOT}/doc/api.html",
+        'lintian_manual'   => "${LINTIAN_BASE}/doc/lintian.html",
+        'lintian_api_docs' => "${LINTIAN_BASE}/doc/api.html",
         'lintian_log_file' => $ARGV[0],
         'resource_dirs'    =>
-          [map { "${LINTIAN_ROOT}/reporting/$_"} qw(images resources)],
+          [map { "${LINTIAN_BASE}/reporting/$_"} qw(images resources)],
     );
 
-    load_templates("$LINTIAN_ROOT/reporting/templates");
+    load_templates("$LINTIAN_BASE/reporting/templates");
 
     # Create lintian.css from a template, install the output file as a resource
     # and discard the original output file.  We do this after installing all
@@ -199,7 +218,7 @@ sub init_globals {
         die("The --reporting-config parameter must point to an existing file\n"
         );
     }
-    $LINTIAN_ROOT = $ENV{'LINTIAN_ROOT'};
+    $LINTIAN_BASE = $ENV{'LINTIAN_BASE'};
 
     $CONFIG = YAML::XS::LoadFile($OPT{'reporting-config'});
     $HARNESS_STATE_DIR = required_cfg_value('storage', 'state-cache');
@@ -209,7 +228,7 @@ sub init_globals {
         $HISTORY = 1;
         $HISTORY_DIR = required_cfg_value('storage', 'historical-data-dir');
         print "Enabling history tracking as ${history_key} is set\n";
-        if (check_path('gnuplot')) {
+        if (length locate_executable('gnuplot')) {
             $GRAPHS = 1;
             print "Enabling graphs (gnuplot is in PATH)\n";
         } else {
@@ -217,7 +236,7 @@ sub init_globals {
             print "No graphs as \"gnuplot\" is not in PATH\n";
         }
         if ($GRAPHS) {
-            if (check_path('scour')) {
+            if (locate_executable('scour')) {
                 $SCOUR_ENABLED = 1;
                 print "Minimizing generated SVG files (scour is in PATH)\n";
             } else {
@@ -243,11 +262,11 @@ sub init_globals {
     $TEMPLATE_CONFIG_VARS->{'LINTIAN_SOURCE'}
       //= 'https://salsa.debian.org/lintian/lintian.git';
 
-    my $profile = dplint::load_profile();
+    my $profile = load_profile();
 
     Lintian::Data->set_vendor($profile);
 
-    $LINTIAN_VERSION = dplint::lintian_version();
+    $LINTIAN_VERSION = $ENV{LINTIAN_VERSION};
     $timestamp = safe_qx(qw(date -u --rfc-822));
     chomp($LINTIAN_VERSION, $timestamp);
 
@@ -621,7 +640,7 @@ sub collect_statistics {
     # all of our statistics.
 
     if (-f $statistics_file) {
-        ($old_statistics) = read_dpkg_control($statistics_file);
+        ($old_statistics) = read_dpkg_control_lc($statistics_file);
     }
     $statistics{'groups-known'} = scalar(keys(%{$state_cache->{'groups'}}));
     $statistics{'groups-backlog'}
@@ -759,7 +778,7 @@ sub update_history_and_make_graphs {
 
         print "Plotting global statistics...\n";
         run_cmd({ 'chdir' => $graph_dir},
-            'gnuplot',"$LINTIAN_ROOT/reporting/graphs/statistics.gpi");
+            'gnuplot',"$LINTIAN_BASE/reporting/graphs/statistics.gpi");
 
         if ($SCOUR_ENABLED) {
             # Do a little "rename" dance to ensure that we keep the
@@ -792,7 +811,7 @@ sub update_history_and_make_graphs {
         if ($GRAPHS) {
             print {$gnuplot_fd} qq{print 'Plotting $tag statistics...'\n};
             print {$gnuplot_fd}
-              qq{call '$LINTIAN_ROOT/reporting/graphs/tags.gpi' '$tag'\n};
+              qq{call '$LINTIAN_BASE/reporting/graphs/tags.gpi' '$tag'\n};
             print {$gnuplot_fd} qq{reset\n};
         }
     }
@@ -1204,6 +1223,36 @@ sub by_tag {
       || $a->{extra}             cmp $b->{extra};
 }
 
+=item split_tag
+
+=cut
+
+{
+    # Matches something like:  (1:2.0-3) [arch1 arch2]
+    # - captures the version and the architectures
+    my $verarchre = qr,(?: \s* \(( [^)]++ )\) \s* \[ ( [^]]++ ) \]),xo;
+    #                             ^^^^^^^^          ^^^^^^^^^^^^
+    #                           ( version   )      [architecture ]
+
+    # matches the full deal:
+    #    1  222 3333  4444444   5555   666  777
+    # -  T: pkg type (version) [arch]: tag [...]
+    #           ^^^^^^^^^^^^^^^^^^^^^
+    # Where the marked part(s) are optional values.  The numbers above
+    # the example are the capture groups.
+    my $TAG_REGEX
+      = qr/([EWIXOPC]): (\S+)(?: (\S+)(?:$verarchre)?)?: (\S+)(?:\s+(.*))?/;
+
+    sub split_tag {
+        my ($tag_input) = @_;
+        my $pkg_type;
+        return unless $tag_input =~ /^${TAG_REGEX}$/;
+        # default value...
+        $pkg_type = $3//'binary';
+        return ($1, $2, $pkg_type, $4, $5, $6, $7);
+    }
+}
+
 1;
 
 # Local Variables:
