@@ -1,5 +1,5 @@
-# Copyright © 2018-2020 Felix Lechner
-# Copyright © 2019 Chris Lamb <lamby@debian.org>
+# Copyright (C) 2018-2020 Felix Lechner
+# Copyright (C) 2019 Chris Lamb <lamby@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, you can find it on the World Wide
-# Web at http://www.gnu.org/copyleft/gpl.html, or write to the Free
+# Web at https://www.gnu.org/copyleft/gpl.html, or write to the Free
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA
 
@@ -57,7 +57,6 @@ use File::stat;
 use List::Util qw(max);
 use Path::Tiny;
 use Text::Template;
-use Try::Tiny;
 use Unicode::UTF8 qw(encode_utf8);
 
 use Lintian::Deb822::Section;
@@ -452,7 +451,8 @@ sub combine_fields {
         $testcase->store(
             $destination,
             join($delimiter,
-                grep { length }($testcase->value($destination),$content)));
+                grep { length }($testcase->value($destination),$content))
+        );
     }
 
     # delete the combined entry if it is empty
